-- 
-- SellPriceTrigger
-- 
-- @Author: Tunis
-- @Date: 04.05.2021
-- @Version: 1.0.0.0
-- 
-- 
-- Changelog:
--
--
--

SellPriceTrigger = {}
SellPriceTrigger.moddir = g_currentModDirectory
SellPriceTrigger.triggers = {} -- All settings/triggers
SellPriceTrigger.NOTIFICATION_TRIGGER = {0.1,0.1,0.5,1} -- Notification color

SellPriceTrigger.debug = false


function SellPriceTrigger:loadMap()

	FSBaseMission.registerActionEvents = Utils.appendedFunction(FSBaseMission.registerActionEvents, SellPriceTrigger.registerActionEventsPlayer)
    BaseMission.removeActionEvents = Utils.appendedFunction(BaseMission.removeActionEvents, SellPriceTrigger.removeActionEventsPlayer)

	-- SaveSettings
	FSBaseMission.saveSavegame = Utils.appendedFunction(FSBaseMission.saveSavegame, SellPriceTrigger.saveCfg)

	--Init
	self.updateTimer = 0.0
	self.updateRun = 3000
	self.updateEvery = 60000
	self.eventName = {}
	self.showHelp = true
	self:fillTypes()
	self:loadCfg()

	--gui
	g_gui:loadProfiles(self.moddir .. "gui/guiProfiles.xml")
	self.gui = g_gui:loadGui(self.moddir .. "gui/SellPriceTriggerGUI.xml", "SellPriceTriggerGUI", SellPriceTriggerGUI:new())

end

function SellPriceTrigger:checkPrices()

	local prices = {}

	for _,station in pairs(g_currentMission.storageSystem.unloadingStations) do
		if station.isSellingPoint ~= nil and station.isSellingPoint == true then
			for fillType,isAccepted in pairs(station.acceptedFillTypes) do
				if isAccepted == true then

					local price = station:getEffectiveFillTypePrice(fillType)*1000
					if prices[fillType] ~= nil then
						if price > prices[fillType].price then
							prices[fillType].price = price
							prices[fillType].stationName = station.stationName
						end
					else
						prices[fillType] = {price=0.0,stationName=""}
						prices[fillType].price = price
						prices[fillType].stationName = station.stationName
					end

				end
			end
		end
	end

	for fillType,trigger in pairs(self.triggers) do

		if prices[fillType] ~= nil then
			if trigger.price > 0 and prices[fillType].price > trigger.price then
				if trigger.over == false or trigger.station ~= prices[fillType].stationName then
					trigger.station = prices[fillType].stationName
					local mess = ('%s %s @ %s'):format(trigger.title, g_i18n:formatMoney(prices[fillType].price), prices[fillType].stationName)
					g_currentMission:addIngameNotification(self.NOTIFICATION_TRIGGER, mess, "SPTupdate")
					self.triggers[fillType].over = true
				end
			else
				if trigger.over == true then
					self.triggers[fillType].over = false
				end
			end

			if prices[fillType].price > trigger.highest then
				trigger.highest = prices[fillType].price
			end
		end

	end
	
	self.updateTimer = 0

end

function SellPriceTrigger:fillTypes()
	for _,filltypes in pairs(g_fillTypeManager.fillTypes) do
		if filltypes.showOnPriceTable ~= nil and filltypes.showOnPriceTable == true then
			if self.debug then
				print("---- ".. tostring(filltypes.index).. " ".. tostring(filltypes.name).. " ".. tostring(filltypes.title))
			end
			self.triggers[filltypes.index] = {index="",name="",title="",price=0,highest=0.0,station="",over=false}
			self.triggers[filltypes.index].index = filltypes.index
			self.triggers[filltypes.index].name = filltypes.name
			self.triggers[filltypes.index].title = filltypes.title
		end
	end
end

function SellPriceTrigger:loadCfg()
	if g_currentMission.missionInfo.savegameDirectory ~= nil then
		local file = g_currentMission.missionInfo.savegameDirectory.. "/sellpricetrigger.xml"
		if fileExists(file) then
			local XML = loadXMLFile("sellPriceTrigger_XML", file, "sellPriceTriggers")
			local index2name = {}
			if self.debug then
				print("---Load XML")
			end

			if getXMLBool(XML, "sellPriceTriggers#showhelp") ~= nil then self.showHelp = getXMLBool(XML, "sellPriceTriggers#showhelp") end
			
			if getXMLBool(XML, "sellPriceTriggers#R") ~= nil and getXMLBool(XML, "sellPriceTriggers#G") ~= nil and getXMLBool(XML, "sellPriceTriggers#B") ~= nil then
				local R = getXMLFloat(XML, "sellPriceTriggers#R")
				local G = getXMLFloat(XML, "sellPriceTriggers#G")
				local B = getXMLFloat(XML, "sellPriceTriggers#B")
				self.NOTIFICATION_TRIGGER = {R,G,B,1}
				if self.debug then
					print("---Load R "..tostring(R).." G "..tostring(G).." B "..tostring(B))
				end
			end

			for fillType,trigger in pairs(self.triggers) do
				index2name[trigger.name] = fillType
			end

			for x=0,1000 do

				local groupNameTag = ("sellPriceTriggers.fillTypes.fillType(%d)"):format(x)
				local name = getXMLString(XML, groupNameTag.. "#name")
				if self.debug then
					print("---XML "..tostring(x).." name "..tostring(name))
				end

				if name == nil then
					break
				end;

				if index2name[name] ~= nil then
					if self.debug then
						print("---XML found name "..tostring(name))
					end
					if getXMLInt(XML, groupNameTag.. "#price") ~= nil then self.triggers[index2name[name]].price = getXMLInt(XML, groupNameTag.. "#price") end
					if getXMLFloat(XML, groupNameTag.. "#highest") ~= nil then self.triggers[index2name[name]].highest = getXMLFloat(XML, groupNameTag.. "#highest") end
				end

			end
		end
	end
end

function SellPriceTrigger:saveCfg()
	if g_currentMission.missionInfo.savegameDirectory ~= nil then
		local file = g_currentMission.missionInfo.savegameDirectory.. "/sellpricetrigger.xml"
		local XML = createXMLFile("sellPriceTrigger_XML", file, "sellPriceTriggers")
		local x = 0
		if SellPriceTrigger.debug then
			print("---Save XML")
		end
		
		setXMLBool(XML, "sellPriceTriggers#showhelp", SellPriceTrigger.showHelp)
		setXMLFloat(XML, "sellPriceTriggers#R", SellPriceTrigger.NOTIFICATION_TRIGGER[1])
		setXMLFloat(XML, "sellPriceTriggers#G", SellPriceTrigger.NOTIFICATION_TRIGGER[2])
		setXMLFloat(XML, "sellPriceTriggers#B", SellPriceTrigger.NOTIFICATION_TRIGGER[3])
		for fillType,trigger in pairs(SellPriceTrigger.triggers) do

			local groupNameTag = ("sellPriceTriggers.fillTypes.fillType(%d)"):format(x)
			setXMLString(XML, groupNameTag.. "#name", trigger.name)
			setXMLInt(XML, groupNameTag.. "#price", trigger.price)
			setXMLFloat(XML, groupNameTag.. "#highest", trigger.highest)
			x=x+1

		end

		saveXMLFile(XML)
	end
end

function SellPriceTrigger:loadSavegame() end
function SellPriceTrigger:saveSavegame() end

function SellPriceTrigger:update(dt)
	self.updateTimer = self.updateTimer + dt * g_currentMission.missionInfo.timeScale
	if self.updateTimer >= self.updateRun then
		self.updateRun = self.updateEvery
		self:checkPrices()
	end
end

function SellPriceTrigger:mouseEvent(posX, posY, isDown, isUp, button) end

function SellPriceTrigger:keyEvent(unicode, sym, modifier, isDown)
	if SellPriceTrigger.debug then
		if Input.isKeyPressed(Input.KEY_f11) then
			SellPriceTrigger:checkPrices()
			g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_OK, "Sell Price Triggers Price check", "SPTupdate")
			return
		end
		if Input.isKeyPressed(Input.KEY_f10) then
			SellPriceTrigger:saveCfg()
			g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_OK, "Sell Price Triggers Saved", "SPTsave")
			return
		end
		if Input.isKeyPressed(Input.KEY_f9) then
			SellPriceTrigger:loadCfg()
			g_currentMission:addIngameNotification(FSBaseMission.INGAME_NOTIFICATION_OK, "Sell Price Triggers Loaded", "SPTload")
			return
		end
	end
end

function SellPriceTrigger:draw() end
function SellPriceTrigger:delete() end
function SellPriceTrigger:deleteMap() end

function SellPriceTrigger:registerActionEventsPlayer()
	-- SellPriceTrigger Setting menu
	local result, eventName = InputBinding.registerActionEvent(g_inputBinding, 'SPT_SHOW_UI', self, SellPriceTrigger.actionEventShowUI, false, true, false, true)
	if result then
		--table.insert(SellPriceTrigger.eventName, eventName)
		SellPriceTrigger.eventName[1] = eventName
        g_inputBinding.events[eventName].displayIsVisible = SellPriceTrigger.showHelp
    end
	if SellPriceTrigger.debug then
		print("--- SellPriceTrigger Debug ... SellPriceTrigger:registerActionEventsPlayer(SellPriceTrigger.eventName)")
		DebugUtil.printTableRecursively(SellPriceTrigger.eventName,"----",0,1)
	end
end

function SellPriceTrigger:removeActionEventsPlayer()
	SellPriceTrigger.eventName = {}
	if SellPriceTrigger.debug then
		print("--- SellPriceTrigger Debug ... SellPriceTrigger:removeActionEventsPlayer(SellPriceTrigger.eventName)")
		DebugUtil.printTableRecursively(SellPriceTrigger.eventName,"----",0,1)
	end
end

function SellPriceTrigger:updateActionEventsPlayer()
	--Update Show Help
	for i=1, #SellPriceTrigger.eventName, 1 do
		if (g_inputBinding.events[SellPriceTrigger.eventName[i]] ~= nil) then
			g_inputBinding.events[SellPriceTrigger.eventName[i]].displayIsVisible = SellPriceTrigger.showHelp
		end
	end
end

function SellPriceTrigger:actionEventShowUI(actionName, keyStatus, inputValue, callbackState, isAnalog)
	if SellPriceTrigger.gui.isOpen then
		SellPriceTrigger.gui:onClickBack()
	elseif g_gui.currentGui == nil then
		g_gui:showGui("SellPriceTriggerGUI")
	end
end

addModEventListener(SellPriceTrigger)